"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DockerManager = void 0;
const util_1 = __importDefault(require("util"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const fs_extra_1 = __importDefault(require("fs-extra"));
const temp_1 = __importDefault(require("temp"));
const logger_1 = require("./logger");
class DockerManager {
    constructor(outputPath = '') {
        this.outputPath = outputPath;
        this.containerId = '';
    }
    async initContainerId() {
        this.containerId = await util_1.default
            .promisify(child_process_1.exec)('docker images validator:latest --format "{{.ID}}"')
            .then(result => result.stdout.trim())
            .catch(reason => {
            logger_1.logger.error(reason.stderr);
            return '';
        });
    }
    async runContainer(schemaPath, schemaName, dataPath, outputPath = this.outputPath) {
        try {
            if (this.containerId.length === 0) {
                await this.initContainerId();
            }
            if (this.containerId.length > 0) {
                // make temp dir for output
                temp_1.default.track();
                const outputDir = temp_1.default.mkdirSync('output');
                const containerOutputPath = path_1.default.join(outputDir, 'output.txt');
                const containerLocationPath = path_1.default.join(outputDir, 'locations.json');
                // copy output files after it finishes
                const runCommand = this.buildRunCommand(schemaPath, dataPath, outputDir, schemaName);
                logger_1.logger.info('Running validator container...');
                logger_1.logger.debug(runCommand);
                return util_1.default
                    .promisify(child_process_1.exec)(runCommand)
                    .then(() => {
                    const containerResult = { pass: true };
                    if (fs_extra_1.default.existsSync(containerOutputPath)) {
                        if (outputPath) {
                            fs_extra_1.default.copySync(containerOutputPath, outputPath);
                        }
                        else {
                            const outputText = fs_extra_1.default.readFileSync(containerOutputPath, 'utf-8');
                            logger_1.logger.info(outputText);
                        }
                    }
                    if (fs_extra_1.default.existsSync(containerLocationPath)) {
                        try {
                            containerResult.locations = fs_extra_1.default.readJsonSync(containerLocationPath);
                        }
                        catch (err) {
                            // something went wrong when reading the location file that the validator produced
                        }
                    }
                    return containerResult;
                })
                    .catch(() => {
                    if (fs_extra_1.default.existsSync(containerOutputPath)) {
                        if (outputPath) {
                            fs_extra_1.default.copySync(containerOutputPath, outputPath);
                        }
                        else {
                            const outputText = fs_extra_1.default.readFileSync(containerOutputPath, 'utf-8');
                            logger_1.logger.info(outputText);
                        }
                    }
                    process.exitCode = 1;
                    return { pass: false };
                });
            }
            else {
                logger_1.logger.error('Could not find a validator docker container.');
                process.exitCode = 1;
                return { pass: false };
            }
        }
        catch (error) {
            logger_1.logger.error(`Error when running validator container: ${error}`);
            process.exitCode = 1;
            return { pass: false };
        }
    }
    buildRunCommand(schemaPath, dataPath, outputDir, schemaName) {
        // figure out mount for schema file
        const absoluteSchemaPath = path_1.default.resolve(schemaPath);
        const schemaDir = path_1.default.dirname(absoluteSchemaPath);
        const schemaFile = path_1.default.basename(absoluteSchemaPath);
        // figure out mount for data file
        const absoluteDataPath = path_1.default.resolve(dataPath);
        const dataDir = path_1.default.dirname(absoluteDataPath);
        const dataFile = path_1.default.basename(absoluteDataPath);
        return `docker run --rm -v "${schemaDir}":/schema/ -v "${dataDir}":/data/ -v "${path_1.default.resolve(outputDir)}":/output/ ${this.containerId} "schema/${schemaFile}" "data/${dataFile}" -o "output/" -s ${schemaName}`;
    }
}
exports.DockerManager = DockerManager;
