"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DownloadManager = void 0;
const axios_1 = __importDefault(require("axios"));
const readline_sync_1 = __importDefault(require("readline-sync"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const temp_1 = __importDefault(require("temp"));
const path_1 = __importDefault(require("path"));
const yauzl_1 = __importDefault(require("yauzl"));
const zlib_1 = require("zlib");
const utils_1 = require("./utils");
const logger_1 = require("./logger");
const promises_1 = require("stream/promises");
const ONE_MEGABYTE = 1024 * 1024;
const DATA_SIZE_WARNING_THRESHOLD = ONE_MEGABYTE * 1024; // 1 gigabyte
class DownloadManager {
    constructor(alwaysYes = false) {
        this.alwaysYes = alwaysYes;
        temp_1.default.track();
        this.folder = temp_1.default.mkdirSync();
    }
    async checkDataUrl(url) {
        try {
            const response = await axios_1.default.head(url);
            if (response.status === 200) {
                let proceedToDownload;
                const contentLength = parseInt(response.headers['content-length']);
                if (this.alwaysYes) {
                    proceedToDownload = true;
                }
                else if (isNaN(contentLength)) {
                    proceedToDownload = readline_sync_1.default.keyInYNStrict('Data file size is unknown. Download this file?');
                }
                else if (contentLength > DATA_SIZE_WARNING_THRESHOLD) {
                    proceedToDownload = readline_sync_1.default.keyInYNStrict(`Data file is ${(contentLength / ONE_MEGABYTE).toFixed(2)} MB in size. Download this file?`);
                }
                else {
                    proceedToDownload = true;
                }
                return proceedToDownload;
            }
            else {
                logger_1.logger.error(`Received unsuccessful status code ${response.status} when checking data file URL: ${url}`);
                return false;
            }
        }
        catch (e) {
            logger_1.logger.error(`Request failed when checking data file URL: ${url}`);
            logger_1.logger.error(e.message);
            return false;
        }
    }
    async downloadDataFile(url, folder = this.folder) {
        const filenameGuess = 'data.json';
        const dataPath = path_1.default.join(folder, filenameGuess);
        return new Promise((resolve, reject) => {
            logger_1.logger.info('Beginning download...\n');
            (0, axios_1.default)({
                method: 'get',
                url: url,
                responseType: 'stream',
                onDownloadProgress: progressEvent => {
                    if (process.stdout.isTTY) {
                        let progressText;
                        if (progressEvent.progress != null) {
                            progressText = `Downloaded ${Math.floor(progressEvent.progress * 100)}% of file (${progressEvent.loaded} bytes)`;
                        }
                        else {
                            progressText = `Downloaded ${progressEvent.loaded} bytes`;
                        }
                        process.stdout.clearLine(0, () => {
                            process.stdout.cursorTo(0, () => {
                                process.stdout.moveCursor(0, -1, () => {
                                    logger_1.logger.info(progressText);
                                });
                            });
                        });
                    }
                }
            })
                .then(response => {
                const contentType = response.headers['content-type'] ?? 'application/octet-stream';
                const finalUrl = response.request.path;
                if ((0, utils_1.isZip)(contentType, finalUrl)) {
                    // zips require additional work to find a JSON file inside
                    const zipPath = path_1.default.join(folder, 'data.zip');
                    const zipOutputStream = fs_extra_1.default.createWriteStream(zipPath);
                    (0, promises_1.pipeline)(response.data, zipOutputStream).then(() => {
                        yauzl_1.default.open(zipPath, { lazyEntries: true, autoClose: false }, (err, zipFile) => {
                            if (err != null) {
                                reject(err);
                            }
                            const jsonEntries = [];
                            zipFile.on('entry', (entry) => {
                                if (entry.fileName.endsWith('.json')) {
                                    jsonEntries.push(entry);
                                }
                                zipFile.readEntry();
                            });
                            zipFile.on('end', () => {
                                logger_1.logger.info('Download complete.');
                                if (jsonEntries.length === 0) {
                                    reject('No JSON file present in zip.');
                                }
                                else {
                                    let chosenEntry;
                                    if (jsonEntries.length === 1) {
                                        chosenEntry = jsonEntries[0];
                                        zipFile.openReadStream(chosenEntry, (err, readStream) => {
                                            const outputStream = fs_extra_1.default.createWriteStream(dataPath);
                                            outputStream.on('finish', () => {
                                                zipFile.close();
                                                resolve(dataPath);
                                            });
                                            outputStream.on('error', () => {
                                                zipFile.close();
                                                reject('Error writing downloaded file.');
                                            });
                                            readStream.pipe(outputStream);
                                        });
                                    }
                                    else {
                                        jsonEntries.sort((a, b) => {
                                            return a.fileName.localeCompare(b.fileName);
                                        });
                                        resolve({ zipFile, jsonEntries, dataPath });
                                    }
                                }
                            });
                            zipFile.readEntry();
                        });
                    });
                }
                else {
                    const outputStream = fs_extra_1.default.createWriteStream(dataPath);
                    outputStream.on('finish', () => {
                        logger_1.logger.info('Download complete.');
                        resolve(dataPath);
                    });
                    outputStream.on('error', () => {
                        reject('Error writing downloaded file.');
                    });
                    if ((0, utils_1.isGzip)(contentType, finalUrl)) {
                        (0, promises_1.pipeline)(response.data, (0, zlib_1.createGunzip)(), outputStream);
                    }
                    else {
                        response.data.pipe(outputStream);
                    }
                }
            })
                .catch(reason => {
                reject('Error downloading data file.');
            });
        });
    }
}
exports.DownloadManager = DownloadManager;
