"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SchemaManager = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const util_1 = __importDefault(require("util"));
const utils_1 = require("./utils");
const temp_1 = __importDefault(require("temp"));
const logger_1 = require("./logger");
const json_node_1 = require("@streamparser/json-node");
const VERSION_TIME_LIMIT = 3000; // three seconds
const BACKWARDS_BYTES = 1000; // read the last 1000 bytes during backwards search
class SchemaManager {
    constructor(repoDirectory = utils_1.config.SCHEMA_REPO_FOLDER, repoUrl = utils_1.config.SCHEMA_REPO_URL) {
        this.repoDirectory = repoDirectory;
        this.repoUrl = repoUrl;
        temp_1.default.track();
        this.storageDirectory = temp_1.default.mkdirSync('schemas');
    }
    get version() {
        return this._version;
    }
    async ensureRepo() {
        if (!fs_extra_1.default.existsSync(path_1.default.join(this.repoDirectory, '.git'))) {
            return util_1.default.promisify(child_process_1.exec)(`git clone ${this.repoUrl} "${this.repoDirectory}"`);
        }
    }
    async useVersion(version) {
        if (this._version === version) {
            return true;
        }
        const tagResult = await util_1.default.promisify(child_process_1.exec)(`git -C "${this.repoDirectory}" tag --list --sort=taggerdate`);
        const tags = tagResult.stdout
            .split('\n')
            .map(tag => tag.trim())
            .filter(tag => tag.length > 0);
        if (tags.includes(version)) {
            await util_1.default.promisify(child_process_1.exec)(`git -C "${this.repoDirectory}" checkout ${version}`);
            this._version = version;
            return true;
        }
        else {
            // we didn't find your tag. maybe you mistyped it, so show the available ones.
            throw new Error(`Could not find a schema version named "${version}". Available versions are:\n${tags.join('\n')}`);
        }
    }
    async useSchema(schemaName) {
        const schemaPath = path_1.default.join(this.storageDirectory, `${schemaName}-${this._version}-${this.strict ? 'strict' : 'loose'}.json`);
        if (fs_extra_1.default.existsSync(schemaPath)) {
            logger_1.logger.debug(`Using cached schema: ${schemaName} ${this._version}`);
            return schemaPath;
        }
        const contentPath = path_1.default.join(this.repoDirectory, 'schemas', schemaName, `${schemaName}.json`);
        if (!fs_extra_1.default.existsSync(contentPath)) {
            return null;
        }
        let schemaContents = fs_extra_1.default.readFileSync(contentPath, 'utf-8');
        if (this.strict) {
            const modifiedSchema = JSON.parse(schemaContents);
            makeSchemaStrict(modifiedSchema);
            schemaContents = JSON.stringify(modifiedSchema);
        }
        fs_extra_1.default.writeFileSync(schemaPath, schemaContents, { encoding: 'utf-8' });
        return schemaPath;
    }
    async determineVersion(dataFile) {
        return new Promise((resolve, reject) => {
            logger_1.logger.debug(`Detecting version for ${dataFile}`);
            const parser = new json_node_1.JSONParser({ paths: ['$.version'], keepStack: false });
            const dataStream = fs_extra_1.default.createReadStream(dataFile);
            let foundVersion = '';
            let forwardReject;
            const forwardSearch = new Promise((resolve, reject) => {
                forwardReject = reject;
                parser.on('data', data => {
                    if (typeof data.value === 'string') {
                        foundVersion = data.value;
                    }
                    dataStream.unpipe();
                    dataStream.destroy();
                    parser.end();
                });
                parser.on('close', () => {
                    if (foundVersion) {
                        logger_1.logger.debug(`Found version: ${foundVersion}`);
                        resolve(foundVersion);
                    }
                    else {
                        reject('No version property available.');
                    }
                });
                parser.on('error', () => {
                    // an error gets thrown when closing the stream early, but that's not an actual problem.
                    // it'll get handled in the close event
                    if (!foundVersion) {
                        reject('Parse error when detecting version.');
                    }
                });
                dataStream.pipe(parser);
            });
            let backwardReject;
            const backwardSearch = new Promise((resolve, reject) => {
                backwardReject = reject;
                fs_extra_1.default.promises
                    .open(dataFile, 'r')
                    .then(async (fileHandle) => {
                    try {
                        const stats = await fileHandle.stat();
                        const lastStuff = await fileHandle.read({
                            position: Math.max(0, stats.size - BACKWARDS_BYTES),
                            length: BACKWARDS_BYTES
                        });
                        if (lastStuff.bytesRead > 0) {
                            const lastText = lastStuff.buffer.toString('utf-8');
                            const versionRegex = /"version"\s*:\s*("(?:\\"|\\\\|[^"])*")/;
                            const versionMatch = lastText.match(versionRegex);
                            if (versionMatch) {
                                const foundVersion = JSON.parse(versionMatch[1]);
                                logger_1.logger.debug(`Found version during backwards search: ${foundVersion}`);
                                resolve(foundVersion);
                            }
                            else {
                                reject('No version found during backwards search');
                            }
                        }
                        else {
                            reject('No bytes read during backwards search');
                        }
                    }
                    finally {
                        fileHandle.close();
                    }
                })
                    .catch(err => {
                    logger_1.logger.debug(`Something went wrong during backwards search: ${err}`);
                    reject('Something went wrong during backwards search');
                });
            });
            const timeLimit = setTimeout(() => {
                logger_1.logger.debug('Could not find version within time limit.');
                if (forwardReject) {
                    forwardReject('Forward timeout cancellation');
                }
                if (backwardReject) {
                    backwardReject('Backward timeout cancellation');
                }
                reject('Could not find version within time limit.');
            }, VERSION_TIME_LIMIT);
            Promise.any([forwardSearch, backwardSearch])
                .then(foundVersion => {
                resolve(foundVersion);
            })
                .catch(() => {
                reject();
            })
                .finally(() => {
                logger_1.logger.debug('Cleaning up from version search.');
                clearTimeout(timeLimit);
                dataStream.unpipe();
                dataStream.destroy();
                parser.end();
            });
        });
    }
}
exports.SchemaManager = SchemaManager;
// note that this only sets additionalProperties to false at the top level, and at the first level of definitions.
// if there are nested definitions, those will not be modified.
function makeSchemaStrict(schema) {
    if (typeof schema === 'object') {
        schema.additionalProperties = false;
        if (schema.definitions != null && typeof schema.definitions === 'object') {
            for (const defKey of Object.keys(schema.definitions)) {
                schema.definitions[defKey].additionalProperties = false;
            }
        }
    }
}
