"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.update = exports.validateFromUrl = exports.validate = void 0;
const util_1 = __importDefault(require("util"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const fs_extra_1 = __importDefault(require("fs-extra"));
const readline_sync_1 = __importDefault(require("readline-sync"));
const utils_1 = require("./utils");
const temp_1 = __importDefault(require("temp"));
const SchemaManager_1 = require("./SchemaManager");
const DockerManager_1 = require("./DockerManager");
const logger_1 = require("./logger");
const DownloadManager_1 = require("./DownloadManager");
async function validate(dataFile, options) {
    // check to see if supplied json file exists
    if (!fs_extra_1.default.existsSync(dataFile)) {
        logger_1.logger.error(`Could not find data file: ${dataFile}`);
        process.exitCode = 1;
        return;
    }
    const schemaManager = new SchemaManager_1.SchemaManager();
    await schemaManager.ensureRepo();
    schemaManager.strict = options.strict;
    schemaManager.shouldDetectVersion = options.schemaVersion == null;
    let versionToUse;
    try {
        const detectedVersion = await schemaManager.determineVersion(dataFile);
        if (!schemaManager.shouldDetectVersion && detectedVersion != options.schemaVersion) {
            logger_1.logger.warn(`Schema version ${options.schemaVersion} was provided, but file indicates it conforms to schema version ${detectedVersion}. ${options.schemaVersion} will be used.`);
        }
        versionToUse = schemaManager.shouldDetectVersion ? detectedVersion : options.schemaVersion;
    }
    catch (err) {
        if (!schemaManager.shouldDetectVersion) {
            versionToUse = options.schemaVersion;
        }
        else {
            // or maybe use the minimum.
            logger_1.logger.error('Data file does not contain version information. Please run again using the --schema-version option to specify a version.');
            process.exitCode = 1;
            return;
        }
    }
    return schemaManager
        .useVersion(versionToUse)
        .then(versionIsAvailable => {
        if (versionIsAvailable) {
            return schemaManager.useSchema(options.target);
        }
    })
        .then(async (schemaPath) => {
        temp_1.default.track();
        if (schemaPath != null) {
            const dockerManager = new DockerManager_1.DockerManager(options.out);
            const downloadManager = new DownloadManager_1.DownloadManager(options.yesAll);
            const containerResult = await dockerManager.runContainer(schemaPath, options.target, dataFile);
            if (containerResult.pass) {
                if (options.target === 'table-of-contents') {
                    const providerReferences = await (0, utils_1.assessTocContents)(containerResult.locations, schemaManager, dockerManager, downloadManager);
                    await (0, utils_1.assessReferencedProviders)(providerReferences, schemaManager, dockerManager, downloadManager);
                }
                else if (options.target === 'in-network-rates' &&
                    containerResult.locations?.providerReference?.length > 0) {
                    await (0, utils_1.assessReferencedProviders)(containerResult.locations.providerReference, schemaManager, dockerManager, downloadManager);
                }
            }
        }
        else {
            logger_1.logger.error('No schema available - not validating.');
            process.exitCode = 1;
        }
        temp_1.default.cleanupSync();
    })
        .catch(err => {
        logger_1.logger.error(err.message);
        process.exitCode = 1;
    });
}
exports.validate = validate;
async function validateFromUrl(dataUrl, options) {
    temp_1.default.track();
    const downloadManager = new DownloadManager_1.DownloadManager(options.yesAll);
    if (await downloadManager.checkDataUrl(dataUrl)) {
        const schemaManager = new SchemaManager_1.SchemaManager();
        await schemaManager.ensureRepo();
        schemaManager.strict = options.strict;
        return schemaManager
            .useVersion(options.schemaVersion)
            .then(versionIsAvailable => {
            if (versionIsAvailable) {
                return schemaManager.useSchema(options.target);
            }
        })
            .then(async (schemaPath) => {
            if (schemaPath != null) {
                const dockerManager = new DockerManager_1.DockerManager(options.out);
                const dataFile = await downloadManager.downloadDataFile(dataUrl);
                if (typeof dataFile === 'string') {
                    const containerResult = await dockerManager.runContainer(schemaPath, options.target, dataFile);
                    if (containerResult.pass) {
                        if (options.target === 'table-of-contents') {
                            const providerReferences = await (0, utils_1.assessTocContents)(containerResult.locations, schemaManager, dockerManager, downloadManager);
                            await (0, utils_1.assessReferencedProviders)(providerReferences, schemaManager, dockerManager, downloadManager);
                        }
                        else if (options.target === 'in-network-rates' &&
                            containerResult.locations?.providerReference?.length > 0) {
                            await (0, utils_1.assessReferencedProviders)(containerResult.locations.providerReference, schemaManager, dockerManager, downloadManager);
                        }
                    }
                    return containerResult;
                }
                else {
                    let continuation = true;
                    // we have multiple files, so let's choose as many as we want
                    while (continuation === true) {
                        const chosenEntry = (0, utils_1.chooseJsonFile)(dataFile.jsonEntries);
                        await (0, utils_1.getEntryFromZip)(dataFile.zipFile, chosenEntry, dataFile.dataPath);
                        await dockerManager.runContainer(schemaPath, options.target, dataFile.dataPath);
                        continuation = readline_sync_1.default.keyInYNStrict('Would you like to validate another file in the ZIP?');
                    }
                    dataFile.zipFile.close();
                }
            }
            else {
                logger_1.logger.error('No schema available - not validating.');
                process.exitCode = 1;
            }
        })
            .catch(err => {
            logger_1.logger.error(err.message);
            process.exitCode = 1;
        });
    }
    else {
        logger_1.logger.info('Exiting.');
        process.exitCode = 1;
    }
}
exports.validateFromUrl = validateFromUrl;
async function update() {
    try {
        // check if the repo exists. if not, clone it. if it exists, fetch updates.
        if (!fs_extra_1.default.existsSync(path_1.default.join(utils_1.config.SCHEMA_REPO_FOLDER, '.git'))) {
            await util_1.default.promisify(child_process_1.exec)(`git clone ${utils_1.config.SCHEMA_REPO_URL} "${utils_1.config.SCHEMA_REPO_FOLDER}"`);
            logger_1.logger.info('Retrieved schemas.');
        }
        else {
            await util_1.default.promisify(child_process_1.exec)(`git -C "${utils_1.config.SCHEMA_REPO_FOLDER}" checkout master && git -C "${utils_1.config.SCHEMA_REPO_FOLDER}" pull --no-rebase -t`);
            logger_1.logger.info('Updated schemas.');
        }
    }
    catch (error) {
        logger_1.logger.error(`Error when updating available schemas: ${error}`);
        process.exitCode = 1;
    }
}
exports.update = update;
