"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateReferencedProviders = exports.assessReferencedProviders = exports.validateTocContents = exports.assessTocContents = exports.appendResults = exports.chooseJsonFile = exports.isZip = exports.isGzip = exports.getEntryFromZip = exports.config = void 0;
const readline_sync_1 = __importDefault(require("readline-sync"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const temp_1 = __importDefault(require("temp"));
const os_1 = require("os");
const logger_1 = require("./logger");
exports.config = {
    AVAILABLE_SCHEMAS: [
        'allowed-amounts',
        'in-network-rates',
        'provider-reference',
        'table-of-contents'
    ],
    SCHEMA_REPO_URL: 'https://github.com/CMSgov/price-transparency-guide.git',
    SCHEMA_REPO_FOLDER: path_1.default.normalize(path_1.default.join(__dirname, '..', 'schema-repo'))
};
async function getEntryFromZip(zipFile, entry, dataPath) {
    return new Promise((resolve, reject) => {
        zipFile.openReadStream(entry, (err, readStream) => {
            if (err) {
                reject(err);
            }
            else {
                const outputStream = fs_extra_1.default.createWriteStream(dataPath);
                outputStream.on('finish', () => {
                    // keep the zipFile open for now, in case we want more entries
                    resolve();
                });
                outputStream.on('error', () => {
                    zipFile.close();
                    reject('Error writing chosen file.');
                });
                readStream.pipe(outputStream);
            }
        });
    });
}
exports.getEntryFromZip = getEntryFromZip;
function isGzip(contentType, url) {
    return (contentType === 'application/gzip' ||
        contentType === 'application/x-gzip' ||
        (contentType === 'application/octet-stream' && /\.gz(\?|$)/.test(url)));
}
exports.isGzip = isGzip;
function isZip(contentType, url) {
    return (contentType === 'application/zip' || /\.zip(\?|$)/.test(url));
}
exports.isZip = isZip;
function chooseJsonFile(entries) {
    // there might be a lot of entries. show ten per page of results
    logger_1.logger.info(`${entries.length} JSON files found within ZIP archive.`);
    const maxPage = Math.floor((entries.length - 1) / 10);
    let currentPage = 0;
    let chosenIndex;
    showMenuOptions(currentPage, maxPage, entries.slice(currentPage * 10, currentPage * 10 + 10).map(ent => ent.fileName));
    readline_sync_1.default.promptCLLoop((command, ...extraArgs) => {
        if (/^n(ext)?$/i.test(command)) {
            if (currentPage < maxPage) {
                currentPage++;
            }
            else {
                logger_1.logger.menu('Already at last page.');
            }
        }
        else if (/^p(revious)?$/i.test(command)) {
            if (currentPage > 0) {
                currentPage--;
            }
            else {
                logger_1.logger.menu('Already at first page.');
            }
        }
        else if (/^go?$/i.test(command)) {
            const targetPage = parseInt(extraArgs[0]);
            if (targetPage > 0 && targetPage <= maxPage + 1) {
                currentPage = targetPage - 1;
            }
            else {
                logger_1.logger.menu("Can't go to that page.");
            }
        }
        else if (/^\d$/.test(command)) {
            chosenIndex = currentPage * 10 + parseInt(command);
            logger_1.logger.menu(`You selected: ${entries[chosenIndex].fileName}`);
            return true;
        }
        else {
            logger_1.logger.menu('Unrecognized command.');
        }
        showMenuOptions(currentPage, maxPage, entries.slice(currentPage * 10, (currentPage + 1) * 10).map(ent => ent.fileName));
    });
    return entries[chosenIndex];
}
exports.chooseJsonFile = chooseJsonFile;
function showMenuOptions(currentPage, maxPage, items) {
    logger_1.logger.menu(`Showing page ${currentPage + 1} of ${maxPage + 1}`);
    items.forEach((item, idx) => {
        logger_1.logger.menu(`(${idx}): ${item}`);
    });
    const commandsToShow = [];
    if (currentPage > 0) {
        commandsToShow.push('(p)revious page');
    }
    if (currentPage < maxPage) {
        commandsToShow.push('(n)ext page');
    }
    if (maxPage > 0) {
        commandsToShow.push('"(g)o X" to jump to a page');
    }
    logger_1.logger.menu(commandsToShow.join(' | '));
}
function appendResults(source, destination, prefixData = '') {
    try {
        const sourceData = fs_extra_1.default.readFileSync(source);
        fs_extra_1.default.appendFileSync(destination, `${prefixData}${sourceData}`);
    }
    catch (err) {
        logger_1.logger.error('Problem copying results to output file', err);
    }
}
exports.appendResults = appendResults;
async function assessTocContents(locations, schemaManager, dockerManager, downloadManager) {
    const totalFileCount = (locations?.inNetwork?.length ?? 0) + (locations?.allowedAmount?.length ?? 0);
    const fileText = totalFileCount === 1 ? 'this file' : 'these files';
    if (totalFileCount > 0) {
        logger_1.logger.info(`Table of contents refers to ${fileText}:`);
        if (locations.inNetwork?.length > 0) {
            logger_1.logger.info('== In-Network Rates ==');
            locations.inNetwork.forEach(inf => logger_1.logger.info(`* ${inf}`));
        }
        if (locations.allowedAmount?.length > 0) {
            logger_1.logger.info('== Allowed Amounts ==');
            locations.allowedAmount.forEach(aaf => logger_1.logger.info(`* ${aaf}`));
        }
        const wantToValidateContents = downloadManager.alwaysYes ||
            readline_sync_1.default.keyInYNStrict(`Would you like to validate ${fileText}?`);
        if (wantToValidateContents) {
            const providerReferences = await validateTocContents(locations.inNetwork ?? [], locations.allowedAmount ?? [], schemaManager, dockerManager, downloadManager);
            return providerReferences;
        }
    }
    return [];
}
exports.assessTocContents = assessTocContents;
async function validateTocContents(inNetwork, allowedAmount, schemaManager, dockerManager, downloadManager) {
    temp_1.default.track();
    let tempOutput = '';
    if (dockerManager.outputPath?.length > 0) {
        tempOutput = path_1.default.join(temp_1.default.mkdirSync('contents'), 'contained-result');
    }
    let providerReferences;
    if (inNetwork.length > 0) {
        if (schemaManager.shouldDetectVersion) {
            providerReferences = await validateInNetworkDetectedVersion(inNetwork, schemaManager, dockerManager, downloadManager, tempOutput);
        }
        else {
            providerReferences = await validateInNetworkFixedVersion(inNetwork, schemaManager, dockerManager, downloadManager, tempOutput);
        }
    }
    if (allowedAmount.length > 0) {
        if (schemaManager.shouldDetectVersion) {
            await validateAllowedAmountsDetectedVersion(allowedAmount, schemaManager, dockerManager, downloadManager, tempOutput);
        }
        else {
            await validateAllowedAmountsFixedVersion(allowedAmount, schemaManager, dockerManager, downloadManager, tempOutput);
        }
    }
    return [...providerReferences.values()];
}
exports.validateTocContents = validateTocContents;
async function validateInNetworkFixedVersion(inNetwork, schemaManager, dockerManager, downloadManager, tempOutput) {
    const providerReferences = new Set();
    await schemaManager.useSchema('in-network-rates').then(async (schemaPath) => {
        if (schemaPath != null) {
            for (const dataUrl of inNetwork) {
                try {
                    if (await downloadManager.checkDataUrl(dataUrl)) {
                        logger_1.logger.info(`File: ${dataUrl}`);
                        const dataPath = await downloadManager.downloadDataFile(dataUrl);
                        if (typeof dataPath === 'string') {
                            // check if detected version matches the provided version
                            // if there's no version property, that's ok
                            await schemaManager
                                .determineVersion(dataPath)
                                .then(detectedVersion => {
                                if (detectedVersion != schemaManager.version) {
                                    logger_1.logger.warn(`Schema version ${schemaManager.version} was provided, but file indicates it conforms to schema version ${detectedVersion}. ${schemaManager.version} will be used.`);
                                }
                            })
                                .catch(() => { });
                            const containedResult = await dockerManager.runContainer(schemaPath, 'in-network-rates', dataPath, tempOutput);
                            if (containedResult.pass &&
                                containedResult.locations?.providerReference?.length > 0) {
                                containedResult.locations.providerReference.forEach(prf => providerReferences.add(prf));
                            }
                            if (tempOutput.length > 0) {
                                appendResults(tempOutput, dockerManager.outputPath, `${dataUrl} - in-network${os_1.EOL}`);
                            }
                        }
                    }
                    else {
                        logger_1.logger.error(`Could not download file: ${dataUrl}`);
                    }
                }
                catch (err) {
                    logger_1.logger.error('Problem validating referenced in-network file', err);
                }
            }
        }
        else {
            logger_1.logger.error('No schema available - not validating.');
        }
    });
    return providerReferences;
}
async function validateInNetworkDetectedVersion(inNetwork, schemaManager, dockerManager, downloadManager, tempOutput) {
    const providerReferences = new Set();
    for (const dataUrl of inNetwork) {
        try {
            if (await downloadManager.checkDataUrl(dataUrl)) {
                logger_1.logger.info(`File: ${dataUrl}`);
                const dataPath = await downloadManager.downloadDataFile(dataUrl);
                if (typeof dataPath === 'string') {
                    const versionToUse = await schemaManager.determineVersion(dataPath);
                    await schemaManager
                        .useVersion(versionToUse)
                        .then(versionIsAvailable => {
                        if (versionIsAvailable) {
                            return schemaManager.useSchema('in-network-rates');
                        }
                        else {
                            return null;
                        }
                    })
                        .then(schemaPath => {
                        if (schemaPath != null) {
                            return dockerManager.runContainer(schemaPath, 'in-network-rates', dataPath, tempOutput);
                        }
                    })
                        .then(containedResult => {
                        if (containedResult.pass &&
                            containedResult.locations?.providerReference?.length > 0) {
                            containedResult.locations.providerReference.forEach(prf => providerReferences.add(prf));
                        }
                        if (tempOutput.length > 0) {
                            appendResults(tempOutput, dockerManager.outputPath, `${dataUrl} - in-network${os_1.EOL}`);
                        }
                    });
                }
            }
        }
        catch (err) {
            logger_1.logger.error('Problem validating referenced in-network file', err);
        }
    }
    return providerReferences;
}
async function validateAllowedAmountsFixedVersion(allowedAmount, schemaManager, dockerManager, downloadManager, tempOutput) {
    await schemaManager.useSchema('allowed-amounts').then(async (schemaPath) => {
        if (schemaPath != null) {
            for (const dataUrl of allowedAmount) {
                try {
                    if (await downloadManager.checkDataUrl(dataUrl)) {
                        logger_1.logger.info(`File: ${dataUrl}`);
                        const dataPath = await downloadManager.downloadDataFile(dataUrl);
                        if (typeof dataPath === 'string') {
                            // check if detected version matches the provided version
                            // if there's no version property, that's ok
                            await schemaManager
                                .determineVersion(dataPath)
                                .then(detectedVersion => {
                                if (detectedVersion != schemaManager.version) {
                                    logger_1.logger.warn(`Schema version ${schemaManager.version} was provided, but file indicates it conforms to schema version ${detectedVersion}. ${schemaManager.version} will be used.`);
                                }
                            })
                                .catch(() => { });
                            await dockerManager.runContainer(schemaPath, 'allowed-amounts', dataPath, tempOutput);
                            if (tempOutput.length > 0) {
                                appendResults(tempOutput, dockerManager.outputPath, `${dataUrl} - allowed-amounts${os_1.EOL}`);
                            }
                        }
                    }
                    else {
                        logger_1.logger.error(`Could not download file: ${dataUrl}`);
                    }
                }
                catch (err) {
                    logger_1.logger.error('Problem validating referenced allowed-amounts file', err);
                }
            }
        }
        else {
            logger_1.logger.error('No schema available - not validating.');
        }
    });
}
async function validateAllowedAmountsDetectedVersion(allowedAmount, schemaManager, dockerManager, downloadManager, tempOutput) {
    for (const dataUrl of allowedAmount) {
        try {
            if (await downloadManager.checkDataUrl(dataUrl)) {
                logger_1.logger.info(`File: ${dataUrl}`);
                const dataPath = await downloadManager.downloadDataFile(dataUrl);
                if (typeof dataPath === 'string') {
                    const versionToUse = await schemaManager.determineVersion(dataPath);
                    await schemaManager
                        .useVersion(versionToUse)
                        .then(versionIsAvailable => {
                        if (versionIsAvailable) {
                            return schemaManager.useSchema('allowed-amounts');
                        }
                        else {
                            return null;
                        }
                    })
                        .then(schemaPath => {
                        if (schemaPath != null) {
                            return dockerManager.runContainer(schemaPath, 'allowed-amounts', dataPath, tempOutput);
                        }
                    })
                        .then(_containedResult => {
                        if (tempOutput.length > 0) {
                            appendResults(tempOutput, dockerManager.outputPath, `${dataUrl} - allowed-amounts${os_1.EOL}`);
                        }
                    });
                }
            }
        }
        catch (err) {
            logger_1.logger.error('Problem validating referenced allowed-amounts file', err);
        }
    }
}
async function assessReferencedProviders(providerReferences, schemaManager, dockerManager, downloadManager) {
    if (providerReferences.length > 0) {
        const fileText = providerReferences.length === 1 ? 'this file' : 'these files';
        if (providerReferences.length === 1) {
            logger_1.logger.info(`In-network file(s) refer to ${fileText}:`);
            logger_1.logger.info('== Provider Reference ==');
            providerReferences.forEach(prf => logger_1.logger.info(`* ${prf}`));
            const wantToValidateProviders = downloadManager.alwaysYes ||
                readline_sync_1.default.keyInYNStrict(`Would you like to validate ${fileText}?`);
            if (wantToValidateProviders) {
                await validateReferencedProviders(providerReferences, schemaManager, dockerManager, downloadManager);
            }
        }
    }
}
exports.assessReferencedProviders = assessReferencedProviders;
async function validateReferencedProviders(providerReferences, schemaManager, dockerManager, downloadManager) {
    temp_1.default.track();
    let tempOutput = '';
    if (dockerManager.outputPath?.length > 0) {
        tempOutput = path_1.default.join(temp_1.default.mkdirSync('providers'), 'contained-result');
    }
    if (providerReferences.length > 0) {
        schemaManager.useSchema('provider-reference').then(async (schemaPath) => {
            if (schemaPath != null) {
                for (const dataUrl of providerReferences) {
                    try {
                        if (await downloadManager.checkDataUrl(dataUrl)) {
                            logger_1.logger.info(`File: ${dataUrl}`);
                            const dataPath = await downloadManager.downloadDataFile(dataUrl);
                            if (typeof dataPath === 'string') {
                                const containedResult = await dockerManager.runContainer(schemaPath, 'provider-reference', dataPath, tempOutput);
                                if (tempOutput.length > 0) {
                                    appendResults(tempOutput, dockerManager.outputPath, `${dataUrl} - provider-reference${os_1.EOL}`);
                                }
                            }
                        }
                        else {
                            logger_1.logger.error(`Could not download file: ${dataUrl}`);
                        }
                    }
                    catch (err) {
                        logger_1.logger.error('Problem validating referenced provider-reference file', err);
                    }
                }
            }
            else {
                logger_1.logger.error('No schema available - not validating.');
            }
        });
    }
}
exports.validateReferencedProviders = validateReferencedProviders;
